@@ -6,12 +6,9 @@
 #include "dailyweatherforecasttest.h"
 #include <kweathercore/hourlyweatherforecast.h>
 
-using namespace KWeatherCore;
+#include <QJsonObject>
 
-void DailyForecastTest::testNull()
-{
-    QCOMPARE(d1.isValid(), true);
-}
+using namespace KWeatherCore;
 
 void DailyForecastTest::testHourlyMerge()
 {
@@ -27,17 +24,15 @@ void DailyForecastTest::testHourlyMerge()
         h.setWindSpeed(5);
         h.setHumidity(0.1);
 
-        d1 += h;
+        d1 += std::move(h);
     }
-    QCOMPARE(d1.isValid(), false);
-
     QCOMPARE(d1.hourlyWeatherForecast().size(), 24);
 }
 void DailyForecastTest::testDailyMerge()
 {
     const auto &vec = d1.hourlyWeatherForecast();
     for (auto &h : vec) {
-        d2 += h;
+        d2 += HourlyWeatherForecast(h);
     }
 
     QCOMPARE(d2.hourlyWeatherForecast().size(), d1.hourlyWeatherForecast().size());
