@@ -10,12 +10,14 @@
 #include "caparea.h"
 #include "capnamedvalue.h"
 #include "capreference.h"
+#include "capresource.h"
 
 #include <KLocalizedString>
 
 #include <QDateTime>
 #include <QDebug>
 #include <QStringTokenizer>
+#include <QUrl>
 
 #include <optional>
 
@@ -58,10 +60,25 @@ static constexpr const MapEntry<CAPAlertInfo::Category> category_map[] = {
     {"Transport", CAPAlertInfo::Category::Transport},
 };
 
-enum class Tags { ALERT, IDENTIFIER, SENDER, SENT_TIME, STATUS, MSG_TYPE, SCOPE, NOTE, INFO, REFERENCES };
+enum class Tags {
+    ADDRESSES,
+    ALERT,
+    CODE,
+    IDENTIFIER,
+    SENDER,
+    SENT_TIME,
+    STATUS,
+    MSG_TYPE,
+    SCOPE,
+    NOTE,
+    INFO,
+    REFERENCES
+};
 
 static constexpr const MapEntry<Tags> tag_map[] = {
+    {"addresses", Tags::ADDRESSES},
     {"alert", Tags::ALERT},
+    {"code", Tags::CODE},
     {"identifier", Tags::IDENTIFIER},
     {"info", Tags::INFO},
     {"msgType", Tags::MSG_TYPE},
@@ -93,10 +110,13 @@ enum class InfoTags {
     RESPONSETYPE,
     CONTACT,
     WEB,
+    AUDIENCE,
+    RESOURCE,
 };
 
 static constexpr const MapEntry<InfoTags> info_tag_map[] = {
     {"area", InfoTags::AREA},
+    {"audience", InfoTags::AUDIENCE},
     {"category", InfoTags::CATEGORY},
     {"certainty", InfoTags::CERTAINITY},
     {"contact", InfoTags::CONTACT},
@@ -110,6 +130,7 @@ static constexpr const MapEntry<InfoTags> info_tag_map[] = {
     {"language", InfoTags::LANGUAGE},
     {"onset", InfoTags::ONSET_TIME},
     {"parameter", InfoTags::PARAMETER},
+    {"resource", InfoTags::RESOURCE},
     {"responseType", InfoTags::RESPONSETYPE},
     {"senderName", InfoTags::SENDERNAME},
     {"severity", InfoTags::SEVERITY},
@@ -211,6 +232,37 @@ static constexpr const MapEntry<CAPAlertInfo::Certainty> certainty_map[] = {
     return res;
 }
 
+[[nodiscard]] static QStringList parseAddresses(const QString &str)
+{
+    QStringList l;
+    bool inQuote = false;
+    qsizetype beginIdx = -1;
+    for (qsizetype i = 0; i < str.size(); ++i) {
+        if (str[i] == '"'_L1) {
+            if (inQuote) {
+                l.push_back(str.mid(beginIdx, i - beginIdx));
+                beginIdx = -1;
+            } else {
+                beginIdx = i + 1;
+            }
+            inQuote = !inQuote;
+        } else if (str[i].isSpace() && !inQuote) {
+            if (beginIdx >= 0) {
+                l.push_back(str.mid(beginIdx, i - beginIdx));
+                beginIdx = -1;
+            }
+        } else if (!inQuote) {
+            if (beginIdx < 0) {
+                beginIdx = i;
+            }
+        }
+    }
+    if (beginIdx >= 0) {
+        l.push_back(str.mid(beginIdx, str.size() - beginIdx));
+    }
+    return l;
+}
+
 CAPParser::CAPParser(const QByteArray &data)
     : m_xml(data)
 {
@@ -288,6 +340,12 @@ CAPAlertMessage CAPParser::parse()
         case Tags::REFERENCES:
             entry.setReferences(parseReferences(m_xml.readElementText()));
             break;
+        case Tags::ADDRESSES:
+            entry.setAddresses(parseAddresses(m_xml.readElementText()));
+            break;
+        case Tags::CODE:
+            entry.addCode(m_xml.readElementText());
+            break;
         default:
             m_xml.skipCurrentElement();
         }
@@ -394,13 +452,19 @@ CAPAlertInfo CAPParser::parseInfo()
                 case InfoTags::WEB:
                     info.setWeb(m_xml.readElementText());
                     break;
+                case InfoTags::AUDIENCE:
+                    info.setAudience(m_xml.readElementText());
+                    break;
                 case InfoTags::EVENTCODE:
                     info.addEventCode(parseNamedValue());
                     break;
+                case InfoTags::RESOURCE:
+                    info.addResource(parseResource());
+                    break;
                 }
             } else {
                 if (m_xml.isStartElement()) {
-                    qWarning() << "unknown element: " << m_xml.name();
+                    qWarning() << "unknown element: " << m_xml.name() << m_xml.lineNumber() << m_xml.columnNumber();
                 }
             }
         }
@@ -478,4 +542,42 @@ std::vector<CAPReference> CAPParser::parseReferences(const QString &refsString)
 
     return refs;
 }
+
+CAPResource CAPParser::parseResource()
+{
+    CAPResource res;
+    const auto elementName = m_xml.name().toString();
+    while (!m_xml.isEndElement() || m_xml.name() != elementName) {
+        m_xml.readNext();
+        if (!m_xml.isStartElement()) {
+            continue;
+        }
+
+        if (m_xml.name() == "resourceDesc"_L1) {
+            res.setDescription(m_xml.readElementText());
+        } else if (m_xml.name() == "mimeType"_L1) {
+            const auto s = m_xml.readElementText();
+            if (const auto idx = s.indexOf(';'_L1); idx > 0) {
+                // we ignore mime type parameters so far
+                res.setMimeTypeName(s.left(idx));
+            } else {
+                res.setMimeTypeName(s);
+            }
+        } else if (m_xml.name() == "size"_L1) {
+            bool ok = false;
+            qsizetype s = m_xml.readElementText().toLongLong(&ok);
+            if (ok) {
+                res.setSize(s);
+            }
+        } else if (m_xml.name() == "uri"_L1) {
+            res.setUri(QUrl(m_xml.readElementText()));
+        } else if (m_xml.name() == "digest"_L1) {
+            res.setDigest(QByteArray::fromHex(m_xml.readElementText().toLatin1()));
+        } else {
+            qDebug() << "unknown resource element:" << m_xml.name();
+        }
+    }
+
+    return res;
+}
 }
